From: Kiyoshi Ueda <k-ueda@ct.jp.nec.com>

This patch adds a new hook for dm path selector: start_io.
Target drivers should call this hook before submitting I/O to
the selected path.
Path selectors can use it to start accounting of the I/O.
(e.g. counting the number of in-flight I/Os.)

The code is based on the patch posted by Stefan Bader:
https://www.redhat.com/archives/dm-devel/2005-October/msg00050.html


Signed-off-by: Stefan Bader <stefan.bader@canonical.com>
Signed-off-by: Kiyoshi Ueda <k-ueda@ct.jp.nec.com>
Signed-off-by: Jun'ichi Nomura <j-nomura@ce.jp.nec.com>
Cc: Vijayakumar Balasubramanian <vijayakumar@hp.com>
---
 drivers/md/dm-mpath.c         |    3 +++
 drivers/md/dm-path-selector.h |    1 +
 2 files changed, 4 insertions(+)

Index: linux-2.6.29-rc7/drivers/md/dm-mpath.c
===================================================================
--- linux-2.6.29-rc7.orig/drivers/md/dm-mpath.c	2009-03-05 12:10:16.000000000 +0000
+++ linux-2.6.29-rc7/drivers/md/dm-mpath.c	2009-03-05 12:10:26.000000000 +0000
@@ -362,6 +362,9 @@ static int map_io(struct multipath *m, s
 
 	mpio->pgpath = pgpath;
 
+	if (r == DM_MAPIO_REMAPPED && pgpath->pg->ps.type->start_io)
+		pgpath->pg->ps.type->start_io(&pgpath->pg->ps, &pgpath->path);
+
 	spin_unlock_irqrestore(&m->lock, flags);
 
 	return r;
Index: linux-2.6.29-rc7/drivers/md/dm-path-selector.h
===================================================================
--- linux-2.6.29-rc7.orig/drivers/md/dm-path-selector.h	2009-02-25 13:42:15.000000000 +0000
+++ linux-2.6.29-rc7/drivers/md/dm-path-selector.h	2009-03-05 12:10:26.000000000 +0000
@@ -75,6 +75,7 @@ struct path_selector_type {
 	int (*status) (struct path_selector *ps, struct dm_path *path,
 		       status_type_t type, char *result, unsigned int maxlen);
 
+	int (*start_io) (struct path_selector *ps, struct dm_path *path);
 	int (*end_io) (struct path_selector *ps, struct dm_path *path);
 };
 
